package org.openxma.dsl.generator.mojo;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.apache.maven.model.FileSet;
import org.apache.maven.plugin.MojoExecutionException;
import org.codehaus.plexus.util.FileUtils;
import org.hibernate.cfg.Configuration;
import org.hibernate.dialect.Dialect;
import org.hibernate.pretty.DDLFormatter;
import org.openxma.dsl.generator.helper.FileUtil;

/**
 * Maven plugin used to invoke the openXMA Ddl generator workflow
 * 
 * @goal generate-ddl
 * @requiresDependencyResolution compile
 * @phase generate-sources
 */
public class DdlGeneratorMojo extends AbstractGeneratorMojo {
    private static final String DEFAULT_FRAGMENT_EXCLUDE = "**/*.fragment.hbm.xml";
    private static final String DEFAULT_GEN_EXCLUDE = "**/*.gen.hbm.xml";
    private static final String DEFAULT_MAPPING_EXCLUDE_PATTERN = "target/**/*.hbm.xml";
	private static final String DEFAULT_MAPPING_INCLUDE_PATTERN = "src/**/*.hbm.xml";
	/**
	 * The location of the workflow descriptor.
	 * 
	 * @parameter default-value="workflow/DdlGenerator.mwe"
	 */
	private String workflowFile;
	/**
	 * {@link org.apache.maven.model.FileSet}
	 * 
	 * @parameter expression="${hibernate.mappings}"
	 */
	private FileSet hibernateMappings;
	/**
	 * @parameter default-value=
	 *            "org.openxma.dsl.platform.hibernate.dialect.ConfigurableOracle10gDialect"
	 */
	private String hibernateDialect;
	/**
	 * @parameter default-value="create_tables.ddl"
	 */
	private String ddlFileName;
	/**
	 * @parameter expression="${basedir}/src/site/database"
	 */
	private File ddlDirectory;

	@Override
	public void doExecute() throws MojoExecutionException {
		extendCurrentClassloader(getClass().getClassLoader());
		String tableCreationScript = createTableCreationScript();
		File tmpDirectory = saveToTmpDirectory(tableCreationScript);
		HashMap<String, String> workflowParameters = new HashMap<String, String>();
		workflowParameters.put("ddlFileName", ddlFileName);
		workflowParameters.put("ddlDirectory", ddlDirectory.toString());
		workflowParameters.put("modelFolder", tmpDirectory.toString());
		runWorkFlow(workflowFile, workflowParameters);
	}

	private File saveToTmpDirectory(String tableCreationScript) throws MojoExecutionException {
		try {
			return FileUtil.writeStringToFile(tableCreationScript, new File(FileUtil.createTempDirectory(), "_" + ddlFileName)).getParentFile();
		} catch (IOException e) {
			throw new MojoExecutionException(e.getMessage(), e);
		}
	}

	private String createTableCreationScript() throws MojoExecutionException {
		Configuration configuration = new Configuration();
		for (File file : getMappingFiles()) {
			configuration.addFile(file);
		}
		String[] creationScript = configuration.generateSchemaCreationScript(createHibernateDialect());
		StringBuilder ddlBuilder = new StringBuilder();
		for (String ddl : creationScript) {
			String string = new DDLFormatter(ddl).format();
			ddlBuilder.append(string);
			ddlBuilder.append(";");
		}
		return ddlBuilder.toString();
	}

	@SuppressWarnings("unchecked")
	private List<File> getMappingFiles() throws MojoExecutionException {
		List<File> files = new ArrayList<File>();
		if (hibernateMappings == null || hibernateMappings.getDirectory() == null) {
			hibernateMappings.setDirectory(project.getBasedir().toString());
			hibernateMappings.addInclude(DEFAULT_MAPPING_INCLUDE_PATTERN);
			hibernateMappings.addExclude(DEFAULT_MAPPING_EXCLUDE_PATTERN);
            hibernateMappings.addExclude(DEFAULT_FRAGMENT_EXCLUDE);
            hibernateMappings.addExclude(DEFAULT_GEN_EXCLUDE);
		}
		File directory = new File(this.hibernateMappings.getDirectory());
		String includes = getCommaSeparatedList(this.hibernateMappings.getIncludes());
		String excludes = getCommaSeparatedList(this.hibernateMappings.getExcludes());
		try {
			files = FileUtils.getFiles(directory, includes, excludes);
		} catch (IOException e) {
			throw new MojoExecutionException(e.getMessage(), e);
		}
		return files;
	}

	private String getCommaSeparatedList(List<String> list) {
		StringBuffer buffer = new StringBuffer();
		for (Iterator<String> iterator = list.iterator(); iterator.hasNext();) {
			String element = iterator.next();
			buffer.append(element.toString());
			if (iterator.hasNext()) {
				buffer.append(",");
			}
		}
		return buffer.toString();
	}

	private Dialect createHibernateDialect() throws MojoExecutionException {
		try {
			Class<?> loadClass = Thread.currentThread().getContextClassLoader().loadClass(hibernateDialect);
			return (Dialect) loadClass.newInstance();
		} catch (Exception e) {
			throw new MojoExecutionException(e.getMessage(), e);
		}
	}

}
